// Copyright 2023 RozK
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, this
// list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation and/or
// other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS” AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
// OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
// OF SUCH DAMAGE.

namespace RK.MicroComponents {

public class MicroEvent<_Param> : MicroActions {

    internal MicroMutex Mutex; // <- MicroActions

    public void AddParallel(MicroFunc<_Param> func) {
        Mutex.Acquire();
        base.AddParallel(func);
        Mutex.Release();
    }

    public void RemoveParallel(MicroFunc<_Param> func) {
        Mutex.Acquire();
        base.RemoveParallel(func);
        Mutex.Release();
    }

    public void AddSync(MicroFunc<_Param> func) {
        Mutex.Acquire();
        base.AddSync(func);
        Mutex.Release();
    }

    public void RemoveSync(MicroFunc<_Param> func) {
        Mutex.Acquire();
        base.RemoveSync(func);
        Mutex.Release();
    }

    public new void RemoveInstance(MicroInstance instance) {
        Mutex.Acquire();
        base.RemoveInstance(instance);
        Mutex.Release();
    }

    public void Terminate() {
        Clear();
        ParallelActions = null;
        SyncActions = null;
    }
}

} // namespace RK.MicroComponents
