
using UnityEngine;
using RK.MicroComponents;

[CreateAssetMenu(fileName = "SimpleTestComponent1", menuName = "MicroComponents/SimpleTestComponent1")]

public sealed class SimpleTestComponent1 : MicroComponent<SimpleTestComponent1, SimpleTestInstance1> {
}

public sealed class SimpleTestInstance1 : MicroInstance<SimpleTestComponent1, SimpleTestInstance1> {

    static SimpleTestInstance1() {
        Configure(
            MicroUpdates.ParallelUpdate,
            parallelBatchSize: 1,
            poolMinimumSize: 5,
            poolMaximumSize: 0,
            poolGranularity: 5);
        UpdateAfter<SimpleTestInstance2>();
    }

    public class OnUpdateParams {
        public readonly SimpleTestInstance1 Sender;
        public readonly int Value;

        public OnUpdateParams(SimpleTestInstance1 sender, int value) {
            Sender = sender;
            Value = value;
        }
    }

    public MicroEvent<OnUpdateParams> OnUpdate;

    private SimpleTestInstance2.Reference _reference2;

    protected sealed override void Initialize() {
        Debug.Log("TestInstance1.Initialize()");
        OnUpdate = new MicroEvent<OnUpdateParams>();
    }

    protected sealed override void Terminate() {
        Debug.Log("TestInstance1.Terminate()");
        OnUpdate.Terminate();
        OnUpdate = null;
    }

    protected sealed override void Start() {
        Debug.Log("TestInstance1.Start()");
        _reference2 = SimpleTestInstance2.From(Behaviour);
    }

    protected sealed override void Stop() {
        Debug.Log("TestInstance1.Stop()");
    }

    protected sealed override void ParallelUpdate() {
        double time = MicroSystem.TimeCache.Time;
        float deltaTime = MicroSystem.TimeCache.DeltaTime;
        Debug.Log($"TestInstance1.ParallelUpdate() : time = {time}, deltaTime = {deltaTime}");
        InvokeEvent(OnUpdate, new OnUpdateParams(this, 42));
        SimpleTestInstance2 instance2 = _reference2.Instance;
        if (instance2 is not null) {
            ParallelCall(instance2.TestParallelAction, "from TestInstance1");
        }
    }

    public void TestParallelAction(string messageFromInstance2) {
        Debug.Log($"TestInstance1.TestAction(): {messageFromInstance2}");
    }
}
