
using UnityEngine;
using RK.MicroComponents;

[CreateAssetMenu(fileName = "SimpleTestComponent2", menuName = "MicroComponents/SimpleTestComponent2")]

public sealed class SimpleTestComponent2 : MicroComponent<SimpleTestComponent2, SimpleTestInstance2> {

    [SerializeField]
    private string _message;

    public string Message { get => _message; }
}

public sealed class SimpleTestInstance2 : MicroInstance<SimpleTestComponent2, SimpleTestInstance2> {

    static SimpleTestInstance2() {
        Configure(
            MicroUpdates.ParallelUpdate,
            poolMinimumSize: 0,
            poolGranularity: 5);
    }

    private SimpleTestInstance1.Reference _reference1;

    protected sealed override void Initialize() {
        Debug.Log("TestInstance2.Initialize()");
    }

    protected sealed override void Terminate() {
        Debug.Log("TestInstance2.Terminate()");
    }

    protected sealed override void Start() {
        Debug.Log("TestInstance2.Start()");
        _reference1 = SimpleTestInstance1.From(Behaviour);
        SimpleTestInstance1 instance1 = _reference1.Instance;
        if (instance1 is not null) {
            instance1.OnUpdate.AddParallel(TestParallelCallback);
            instance1.OnUpdate.AddSync(TestSyncCallback);
        }
    }

    protected sealed override void Stop() {
        Debug.Log("TestInstance2.Stop()");
        SimpleTestInstance1 instance1 = _reference1.Instance;
        if (instance1 is not null) {
            instance1.OnUpdate.RemoveInstance(this);
        }
    }

    protected sealed override void ParallelUpdate() {
        string message = Component.Message;
        Debug.Log($"TestInstance2.ParallelUpdate(): {message}");
        SimpleTestInstance1 instance1 = _reference1.Instance;
        if (instance1 is not null) {
            ParallelCall(instance1.TestParallelAction, "from TestInstance2");
        }
    }

    public void TestParallelAction(string messageFromInstance1) {
        Debug.Log($"TestInstance2.TestAction(): {messageFromInstance1}");
    }

    private void TestParallelCallback(SimpleTestInstance1.OnUpdateParams param) {
        Debug.Log($"TestInstance2.TestParallelEvent() : {param.Value}");
    }

    private void TestSyncCallback(SimpleTestInstance1.OnUpdateParams param) {
        Debug.Log($"TestInstance2.TestSyncEvent() : {param.Value}");
    }
}
